/*
 * Programming in Java, Jacobs University, 2015
 *
 * This file demonstrates how an iterator can be implemented that can
 * be used to iterate over the lines of a UTF-8 encoded text file in a
 * very compact manner.
 */

import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.UnsupportedEncodingException;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.util.Iterator;
import java.util.logging.Level;
import java.util.logging.Logger;

public class FileScanner implements Iterable<String>
{
    private BufferedReader aReader;

    public FileScanner(String filename)
	throws FileNotFoundException, UnsupportedEncodingException
    {
	FileInputStream fis = new FileInputStream(filename);
	InputStreamReader isr = new InputStreamReader(fis, "UTF-8");
	aReader = new BufferedReader(isr);
    }

    public Iterator<String> iterator()
    {
	return new Iterator<String>() {
	    
	    private String line = getLine();

	    public boolean hasNext()
	    {
		return line != null;
	    }
	    
	    public String next()
	    {
		String retval = line;
		line = getLine();
		return retval;
	    }
	    
	    public void remove()
	    {
		throw new UnsupportedOperationException();
	    }
	    
	    private String getLine()
	    {
		String line = null;
		try {
		    line = aReader.readLine();
		} catch (IOException e) {
		    line = null;
		    log(Level.INFO, e);
		    try {
			aReader.close();
		    } catch (IOException x) {
			log(Level.INFO, x);
		    }
		}
		return line;
	    }

	    private void log(Level level, Exception exception)
	    {
		Logger l = Logger.getLogger(Logger.GLOBAL_LOGGER_NAME);
		l.log(level, exception.getMessage());
	    }
	};
    }

    public static void main(String[] args) throws IOException
    {
	for (String s: args) {
	    for (String line : new FileScanner(s)) {
		System.out.println(line);
	    }
	}
    }
}
